@@ -4,6 +4,7 @@
 # License: The MIT License (MIT)
 
 import sys
+import functools
 from gzip import GzipFile
 import zlib
 from io import BytesIO
@@ -11,7 +12,7 @@ from io import BytesIO
 from collections import defaultdict
 
 import brotli
-from flask import request, current_app
+from flask import request, after_this_request, current_app
 
 
 if sys.version_info[:2] == (2, 6):
@@ -159,21 +160,22 @@ class Compress(object):
 
     def after_request(self, response):
         app = self.app or current_app
-        accept_encoding = request.headers.get('Accept-Encoding', '')
 
+        accept_encoding = request.headers.get('Accept-Encoding', '')
         chosen_algorithm = self._choose_compress_algorithm(accept_encoding)
 
-        if (response.mimetype not in app.config['COMPRESS_MIMETYPES'] or
-            chosen_algorithm is None or
-            not 200 <= response.status_code < 300 or
+        if (chosen_algorithm is None or
+            response.mimetype not in app.config["COMPRESS_MIMETYPES"] or
+            response.status_code < 200 or
+            response.status_code >= 300 or
+            "Content-Encoding" in response.headers or
             (response.content_length is not None and
-             response.content_length < app.config['COMPRESS_MIN_SIZE']) or
-            'Content-Encoding' in response.headers):
+             response.content_length < app.config["COMPRESS_MIN_SIZE"])):
             return response
 
         response.direct_passthrough = False
 
-        if self.cache:
+        if self.cache is not None:
             key = self.cache_key(request)
             compressed_content = self.cache.get(key)
             if compressed_content is None:
@@ -187,15 +189,31 @@ class Compress(object):
         response.headers['Content-Encoding'] = chosen_algorithm
         response.headers['Content-Length'] = response.content_length
 
+        # "123456789"   => "123456789:gzip"   - A strong ETag validator
+        # W/"123456789" => W/"123456789:gzip" - A weak ETag validator
+        etag = response.headers.get('ETag')
+        if etag:
+            response.headers['ETag'] = '{0}:{1}"'.format(etag[:-1], chosen_algorithm)
+
         vary = response.headers.get('Vary')
-        if vary:
-            if 'accept-encoding' not in vary.lower():
-                response.headers['Vary'] = '{}, Accept-Encoding'.format(vary)
-        else:
+        if not vary:
             response.headers['Vary'] = 'Accept-Encoding'
+        elif 'accept-encoding' not in vary.lower():
+            response.headers['Vary'] = '{}, Accept-Encoding'.format(vary)
 
         return response
 
+    def compressed(self):
+        def decorator(f):
+            @functools.wraps(f)
+            def decorated_function(*args, **kwargs):
+                @after_this_request
+                def compressor(response):
+                    return self.after_request(response)
+                return f(*args, **kwargs)
+            return decorated_function
+        return decorator
+
     def compress(self, app, response, algorithm):
         if algorithm == 'gzip':
             gzip_buffer = BytesIO()
